@@ -16,7 +16,7 @@
 
 #include "common/menu.h"
 #include "linux/fbdev.h"
-#include "linux/oshide.h"
+#include "linux/xenv.h"
 #include "plugin_lib.h"
 #include "pl_gun_ts.h"
 #include "omap.h"
@@ -90,10 +90,15 @@ int omap_enable_layer(int enabled)
 
 void plat_video_menu_enter(int is_rom_loaded)
 {
+	int dummy;
+
 	g_menuscreen_ptr = vout_fbdev_resize(main_fb,
 		g_menuscreen_w, g_menuscreen_h, 16, 0, 0, 0, 0, 3);
 	if (g_menuscreen_ptr == NULL)
 		fprintf(stderr, "warning: vout_fbdev_resize failed\n");
+
+	// hmh
+	xenv_update(&dummy);
 }
 
 void plat_video_menu_begin(void)
@@ -116,11 +121,18 @@ void plat_video_menu_leave(void)
 		fprintf(stderr, "warning: vout_fbdev_resize failed\n");
 }
 
+void plat_minimize(void)
+{
+	omap_enable_layer(0);
+	xenv_minimize();
+	omap_enable_layer(1);
+}
+
 void plat_step_volume(int is_up)
 {
 }
 
-void plat_trigger_vibrate(void)
+void plat_trigger_vibrate(int is_strong)
 {
 }
 
@@ -153,7 +165,7 @@ void plat_init(void)
 		exit(1);
 	}
 
-	oshide_init();
+	xenv_init("PCSX-ReARMed");
 
 	w = h = 0;
 	main_fb = vout_fbdev_init(main_fb_name, &w, &h, 16, 2);
@@ -165,9 +177,11 @@ void plat_init(void)
 	g_menuscreen_w = w;
 	g_menuscreen_h = h;
 	g_menuscreen_ptr = vout_fbdev_flip(main_fb);
+	pl_rearmed_cbs.screen_w = w;
+	pl_rearmed_cbs.screen_h = h;
 
 	w = 640;
-	h = 512; // ??
+	h = 512;
 	layer_fb = vout_fbdev_init(layer_fb_name, &w, &h, 16, 3);
 	if (layer_fb == NULL) {
 		fprintf(stderr, "couldn't init fb: %s\n", layer_fb_name);
@@ -198,6 +212,6 @@ void plat_finish(void)
 	omap_enable_layer(0);
 	vout_fbdev_finish(layer_fb);
 	vout_fbdev_finish(main_fb);
-	oshide_finish();
+	xenv_finish();
 }
 
