@@ -5,6 +5,7 @@
 #include <unistd.h>
 #include <hildon/hildon.h>
 #include <string.h>
+#include <pthread.h>
 #include "plugin_lib.h"
 
 #include "main.h"
@@ -22,6 +23,8 @@
 #define D_HEIGHT		480
 
 osso_context_t* osso;
+extern int bKeepDisplayOn;
+static pthread_t display_thread = (pthread_t)-1;
 int g_layer_x = (X_RES - D_WIDTH) / 2;
 int g_layer_y = (Y_RES - D_HEIGHT) / 2;
 int g_layer_w = D_WIDTH, g_layer_h = D_HEIGHT;
@@ -31,7 +34,6 @@ static HildonAnimationActor *actor;
 static GtkWidget *window, *drawing;
 
 static int pl_buf_w, pl_buf_h;
-static int sens, y_def;
 int keymap[65536];
 
 // map psx4m compatible keymap to PSX keys
@@ -66,6 +68,22 @@ window_key_proxy(GtkWidget *widget,
 	return key_press_event(event->hardware_keycode, event->type == GDK_KEY_PRESS?1:(event->type == GDK_KEY_RELEASE?2:0) );
 }
 
+static void displayThread(void *arg)
+{
+	DBusConnection* system_bus = (DBusConnection*)osso_get_sys_dbus_connection(osso);
+	DBusMessage* msg = dbus_message_new_method_call("com.nokia.mce", 
+												    "/com/nokia/mce/request",
+												    "com.nokia.mce.request", 
+												    "req_display_blanking_pause");
+	if (msg && system_bus) { 
+		while (1) {
+			dbus_connection_send(system_bus, msg, NULL);
+			dbus_connection_flush(system_bus);
+			usleep(4000000);
+		}
+	}
+}
+
 void show_notification(char* text)
 {
 	DBusConnection* session_bus = (DBusConnection*)osso_get_dbus_connection(osso);
@@ -117,10 +135,10 @@ void change_disc()
 
 			set_cd_image(filename);
 			if (ReloadCdromPlugin() < 0)
-				printf("failed to load cdr plugin\n");
+				printf("Failed to load cdr plugin\n");
 			
 			if (CDR_open() < 0)
-				printf("failed to open cdr plugin\n");		
+				printf("Failed to open cdr plugin\n");		
 
 			strcpy(file_name, strrchr(filename,'/'));
 
@@ -225,6 +243,12 @@ void plat_finish()
 	hildon_quit();
 }
 
+void set_accel_multipliers()
+{
+	accelOptions.xMultiplier = 255.0 / ( (accelOptions.maxValue - accelOptions.sens) * 2.0);
+	accelOptions.yMultiplier = 255.0 / ( (accelOptions.maxValue - accelOptions.sens) * 2.0);
+}
+
 void maemo_init(int *argc, char ***argv)
 {
 	osso = osso_initialize("pcsxrearmed", PACKAGE_VERSION, FALSE, NULL);
@@ -243,14 +267,6 @@ void maemo_init(int *argc, char ***argv)
 		fclose(pFile);
 	}
 
-	pFile = fopen("/opt/psx4m/config", "r");
-	if (NULL != pFile) {
-		fscanf(pFile, "%d %d",&sens,&y_def);
-		fclose(pFile);
-	} else {
-		sens=150;
-		y_def=500; //near 45 degrees =)
-	}
 	if (!(g_maemo_opts&8)){
 		gtk_init (argc, argv);
 
@@ -280,6 +296,13 @@ void maemo_init(int *argc, char ***argv)
 		gtk_widget_show_all (GTK_WIDGET (actor));
 		gtk_widget_show_all (GTK_WIDGET (window));
 	}
+
+	set_accel_multipliers();
+
+	if (bKeepDisplayOn){
+		if (pthread_create(&display_thread, NULL, displayThread, NULL))
+			printf("Failed to create display thread.\n");
+	}
 }
 
 void menu_loop(void)
@@ -322,21 +345,48 @@ void *hildon_flip(void)
 
 	// process accelometer
 	if (g_maemo_opts & 4) {
-		int x, y, z;
+		float x, y, z;
 		FILE* f = fopen( "/sys/class/i2c-adapter/i2c-3/3-001d/coord", "r" );
 		if( !f ) {printf ("err in accel"); exit(1);}
-		fscanf( f, "%d %d %d", &x, &y, &z );
+		fscanf( f, "%f %f %f", &x, &y, &z );
 		fclose( f );
 
-		if( x > sens ) in_keystate |= 1 << DKEY_LEFT;
-		else if( x < -sens ) in_keystate |= 1 << DKEY_RIGHT;
-		else {in_keystate &= ~(1 << DKEY_LEFT);in_keystate &= ~(1 << DKEY_RIGHT);}
+		if (in_type1 == PSE_PAD_TYPE_ANALOGPAD){
+			if (x > accelOptions.maxValue) x = accelOptions.maxValue;
+			else if (x < -accelOptions.maxValue) x = -accelOptions.maxValue;
 
-		y+=y_def;
-		if( y > sens )in_keystate |= 1 << DKEY_UP;
-		else if( y < -sens ) in_keystate |= 1 << DKEY_DOWN;
-		else {in_keystate &= ~(1 << DKEY_DOWN);in_keystate &= ~(1 << DKEY_UP);}
+			const int maxValue = accelOptions.maxValue - accelOptions.sens;
+			if(x > accelOptions.sens){
+				x -= accelOptions.sens;
+				in_a1[0] = (-x + maxValue ) *  accelOptions.xMultiplier;
+			}else if (x < -accelOptions.sens){
+				x += accelOptions.sens;
+				in_a1[0] = (-x + maxValue ) *  accelOptions.xMultiplier;
+			}else in_a1[0] = 127;
 
+			y += accelOptions.y_def;
+			if (y > accelOptions.maxValue) y = accelOptions.maxValue;
+			else if (y < -accelOptions.maxValue) y = -accelOptions.maxValue;
+
+			if(y > accelOptions.sens){
+				y -= accelOptions.sens;
+				in_a1[1] = (-y + maxValue ) *  accelOptions.yMultiplier;
+			}else if (y < -accelOptions.sens){
+				y += accelOptions.sens;
+				in_a1[1] = (-y + maxValue ) *  accelOptions.yMultiplier;
+			}else in_a1[1] = 127;
+
+			//printf("x: %i y: %i\n", in_a1[0], in_a1[1]);
+		}else{
+			if( x > accelOptions.sens ) in_keystate |= 1 << DKEY_LEFT;
+			else if( x < -accelOptions.sens ) in_keystate |= 1 << DKEY_RIGHT;
+			else {in_keystate &= ~(1 << DKEY_LEFT);in_keystate &= ~(1 << DKEY_RIGHT);}
+
+			y += accelOptions.y_def;
+			if( y > accelOptions.sens )in_keystate |= 1 << DKEY_UP;
+			else if( y < -accelOptions.sens ) in_keystate |= 1 << DKEY_DOWN;
+			else {in_keystate &= ~(1 << DKEY_DOWN);in_keystate &= ~(1 << DKEY_UP);}
+		}
 	}
 
 	if (g_maemo_opts & 8)
