@@ -19,13 +19,18 @@
 #include "../plugins/dfinput/main.h"
 #include "maemo_common.h"
 
+extern int cycle_multiplier;
+extern int in_type1, in_type2;
+
 // sound plugin
 extern int iUseTimer;
 extern int iUseReverb;
 extern int iUseInterpolation;
 
+accel_option accelOptions;
 int g_opts = 0;
 int g_maemo_opts;
+int bKeepDisplayOn = FALSE;
 char file_name[MAXPATHLEN];
 
 enum sched_action emu_action;
@@ -59,6 +64,13 @@ void PrintHelp()
 	printf("                    1=Set to 1\n");
 	printf("                    ...\n");
 	printf("  -accel         : Enable accelerometer\n");
+	printf("  -analog        : Use analog pad for accel\n");
+	printf("  -sens VALUE    : Set accelerometer sens [0-1000]\n");
+    printf("                   (Default 150)\n");
+	printf("  -ydef VALUE    : Set accelerometer y zero [0-1000]\n");
+    printf("                   (Default 500)\n");
+	printf("  -max VALUE     : Set accelerometer max value[0-1000]\n");
+    printf("                   (Default 500)\n");
 	printf("  -sputhreaded   : Use a separate thread for sound\n");
 	printf("  -nosound       : No sound output\n");
 	printf("  -bdir PATH     : Set the bios path\n");
@@ -79,6 +91,11 @@ void PrintHelp()
 	printf("                   -1=Auto (Default)\n");
 	printf("                    0=NTSC\n");
 	printf("                    1=PAL\n");
+	printf("  -cpuclock VALUE: PSX CPU clock %% [1-500]\n");
+    printf("                   (Default 50)\n");
+	printf("  -displayon     : Prevent display from blanking\n");
+    printf("                   (Default disabled)\n");
+
 
 	printf("\nGPU Options:\n");
 	printf("  -gles          : Use the GLES plugin (gpuGLES.so)\n");
@@ -157,6 +174,13 @@ int maemo_main(int argc, char **argv)
     iUseReverb = 0;
     iUseInterpolation = 0;
 
+	in_type1 = PSE_PAD_TYPE_STANDARD;
+	in_type2 = PSE_PAD_TYPE_STANDARD;
+
+	accelOptions.sens     = 150;
+	accelOptions.y_def	  = 500;
+	accelOptions.maxValue = 500.0;
+
 	// read command line options
 	for (i = 1; i < argc; i++) {
 		     if (!strcmp(argv[i], "-psxout")) Config.PsxOut = 1;
@@ -217,6 +241,13 @@ int maemo_main(int argc, char **argv)
 		else if (!strcmp(argv[i], "-mcd1")) 	            sprintf(Config.Mcd1, "%s", argv[++i]);
 		else if (!strcmp(argv[i], "-mcd2")) 	            sprintf(Config.Mcd2, "%s", argv[++i]);
 
+		else if (!strcmp(argv[i], "-cpuclock")) 	        cycle_multiplier = 10000 / atol(argv[++i]);
+		else if (!strcmp(argv[i], "-analog")) 	            in_type1 = PSE_PAD_TYPE_ANALOGPAD;
+		else if (!strcmp(argv[i], "-sens")) 				accelOptions.sens = atol(argv[++i]);
+		else if (!strcmp(argv[i], "-ydef")) 				accelOptions.y_def = atol(argv[++i]);
+		else if (!strcmp(argv[i], "-max")) 				    accelOptions.maxValue = atol(argv[++i]);
+		else if (!strcmp(argv[i], "-displayon")) 		    bKeepDisplayOn = TRUE;
+
 		else if (!strcmp(argv[i], "-spu_reverb"))           { if (atol(argv[++i]) > 0) iUseReverb = 2; }
 		else if (!strcmp(argv[i], "-spu_interpolation")) 	iUseInterpolation = atol(argv[++i]);
 /*
@@ -294,7 +325,7 @@ int maemo_main(int argc, char **argv)
 	// If a state has been specified, then load that
 	if (loadst) {
 		int ret = emu_load_state(loadst - 1);
-		printf("%s state %d\n", ret ? "failed to load" : "loaded", loadst);
+		printf("%s state %d\n", ret ? "Failed to load" : "Loaded", loadst);
 		state_slot = loadst - 1;
 	}
 
