@@ -16,6 +16,7 @@
 
 #include "common.h"
 
+#ifndef command_lengths
 const u8 command_lengths[256] =
 {
 	0,  0,  2,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,   // 00
@@ -24,7 +25,7 @@ const u8 command_lengths[256] =
 	5,  5,  5,  5,  8,  8,  8,  8,  7,  7,  7,  7,  11, 11, 11, 11,  // 30
 	2,  2,  2,  2,  2,  2,  2,  2,  3,  3,  3,  3,  3,  3,  3,  3,   // 40
 	3,  3,  3,  3,  3,  3,  3,  3,  4,  4,  4,  4,  4,  4,  4,  4,   // 50
-	2,  2,  2,  2,  3,  3,  3,  3,  1,  1,  1,  1,  1,  1,  1,  1,   // 60
+	2,  2,  2,  2,  3,  3,  3,  3,  1,  1,  1,  1,  0,  0,  0,  0,   // 60
 	1,  1,  1,  1,  2,  2,  2,  2,  1,  1,  1,  1,  2,  2,  2,  2,   // 70
 	3,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,   // 80
 	0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,   // 90
@@ -35,6 +36,7 @@ const u8 command_lengths[256] =
 	0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,   // e0
 	0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0    // f0
 };
+#endif
 
 void update_texture_ptr(psx_gpu_struct *psx_gpu)
 {
@@ -197,12 +199,17 @@ void set_triangle_color(psx_gpu_struct *psx_gpu, u32 triangle_color)
   get_vertex_data_xy(vertex_number, offset16);                                 \
   set_vertex_color_constant(vertex_number, color)                              \
 
+#ifndef SET_Ex
+#define SET_Ex(r, v)
+#endif
+
 vertex_struct vertexes[4] __attribute__((aligned(32)));
 
-void gpu_parse(psx_gpu_struct *psx_gpu, u32 *list, u32 size)
+u32 gpu_parse(psx_gpu_struct *psx_gpu, u32 *list, u32 size, u32 *last_command)
 {
-  u32 current_command, command_length;
-  
+  u32 current_command = 0, command_length;
+
+  u32 *list_start = list;
   u32 *list_end = list + (size / 4);
 
   for(; list < list_end; list += 1 + command_length)
@@ -210,16 +217,65 @@ void gpu_parse(psx_gpu_struct *psx_gpu, u32 *list, u32 size)
   	s16 *list_s16 = (void *)list;
   	current_command = *list >> 24;
   	command_length = command_lengths[current_command];
-  
+  	if (list + 1 + command_length > list_end) {
+  	  current_command = (u32)-1;
+  	  break;
+  	}
+
   	switch(current_command)
   	{
   		case 0x00:
   			break;
   
   		case 0x02:
-        render_block_fill(psx_gpu, list[0] & 0xFFFFFF, list_s16[2] & 0x3FF,
-         list_s16[3] & 0x1FF, list_s16[4] & 0x3FF, list_s16[5] & 0x1FF);
+      {
+        u32 x = list_s16[2] & 0x3FF;
+        u32 y = list_s16[3] & 0x1FF;
+        u32 width = list_s16[4] & 0x3FF;
+        u32 height = list_s16[5] & 0x1FF;
+        u32 color = list[0] & 0xFFFFFF;
+
+        x &= ~0xF;
+        width = ((width + 0xF) & ~0xF);
+
+        if((x + width) > 1024)
+        {
+          u32 width_a = 1024 - x;
+          u32 width_b = width - width_a;
+
+          if((y + height) > 512)
+          {
+            u32 height_a = 512 - y;
+            u32 height_b = height - height_a;
+
+            render_block_fill(psx_gpu, color, x, y, width_a, height_a);
+            render_block_fill(psx_gpu, color, 0, y, width_b, height_a);
+            render_block_fill(psx_gpu, color, x, 0, width_a, height_b);
+            render_block_fill(psx_gpu, color, 0, 0, width_b, height_b);
+          }
+          else
+          {
+            render_block_fill(psx_gpu, color, x, y, width_a, height);
+            render_block_fill(psx_gpu, color, 0, y, width_b, height);
+          }
+        }
+        else
+        {
+          if((y + height) > 512)
+          {
+            u32 height_a = 512 - y;
+            u32 height_b = height - height_a;
+
+            render_block_fill(psx_gpu, color, x, y, width, height_a);
+            render_block_fill(psx_gpu, color, x, 0, width, height_b);
+          }
+          else
+          {
+            render_block_fill(psx_gpu, color, x, y, width, height);
+          }
+        }
   			break;
+      }
   
   		case 0x20 ... 0x23:
       {
@@ -228,7 +284,7 @@ void gpu_parse(psx_gpu_struct *psx_gpu, u32 *list, u32 size)
         get_vertex_data_xy(0, 2);
         get_vertex_data_xy(1, 4);
         get_vertex_data_xy(2, 6);
-  
+          
         render_triangle(psx_gpu, vertexes, current_command);
   			break;
       }
@@ -359,7 +415,8 @@ void gpu_parse(psx_gpu_struct *psx_gpu, u32 *list, u32 size)
         while(1)
         {
           xy = *list_position;
-          if(xy == 0x55555555)
+
+          if((xy & 0xF000F000) == 0x50005000)
             break;
 
           vertexes[0] = vertexes[1];
@@ -370,6 +427,9 @@ void gpu_parse(psx_gpu_struct *psx_gpu, u32 *list, u32 size)
           list_position++;
           num_vertexes++;
 
+          if(list_position > list_end)
+            break;
+
           render_line(psx_gpu, vertexes, current_command, list[0]);
         }
 
@@ -413,7 +473,7 @@ void gpu_parse(psx_gpu_struct *psx_gpu, u32 *list, u32 size)
         while(1)
         {
           color = list_position[0];
-          if(color == 0x55555555)
+          if((color & 0xF000F000) == 0x50005000)
             break;
 
           xy = list_position[1];
@@ -429,6 +489,9 @@ void gpu_parse(psx_gpu_struct *psx_gpu, u32 *list, u32 size)
           list_position += 2;
           num_vertexes++;
 
+          if(list_position > list_end)
+            break;
+
           render_line(psx_gpu, vertexes, current_command, 0);
         }
 
@@ -536,7 +599,12 @@ void gpu_parse(psx_gpu_struct *psx_gpu, u32 *list, u32 size)
         render_block_move(psx_gpu, list_s16[2] & 0x3FF, list_s16[3] & 0x1FF,
          list_s16[4] & 0x3FF, list_s16[5] & 0x1FF, list_s16[6], list_s16[7]);
   			break;
-  
+ 
+#ifdef PCSX
+  		case 0xA0:          //  sys -> vid
+  		case 0xC0:          //  vid -> sys
+  			goto breakloop;
+#else
   		case 0xA0:          //  sys -> vid
       {
         u32 load_x = list_s16[2] & 0x3FF;
@@ -546,23 +614,29 @@ void gpu_parse(psx_gpu_struct *psx_gpu, u32 *list, u32 size)
         u32 load_size = load_width * load_height;
   
         command_length += load_size / 2;
-  
+
+        if(load_size & 1)
+          command_length++;
+
         render_block_copy(psx_gpu, (u16 *)&(list_s16[6]), load_x, load_y,
          load_width, load_height, load_width);
   			break;
       }
-  
+
   		case 0xC0:          //  vid -> sys
   			break;
-  
+#endif
+
   		case 0xE1:
         set_texture(psx_gpu, list[0] & 0x1FF);
+
         if(list[0] & (1 << 9))
           psx_gpu->render_state_base |= RENDER_STATE_DITHER;
         else
           psx_gpu->render_state_base &= ~RENDER_STATE_DITHER;
 
         psx_gpu->display_area_draw_enable = (list[0] >> 10) & 0x1;
+  			SET_Ex(1, list[0]);
   			break;
   
   		case 0xE2:
@@ -595,6 +669,7 @@ void gpu_parse(psx_gpu_struct *psx_gpu, u32 *list, u32 size)
 
           update_texture_ptr(psx_gpu);
         }
+        SET_Ex(2, list[0]);
         break;
   		}
   
@@ -608,6 +683,7 @@ void gpu_parse(psx_gpu_struct *psx_gpu, u32 *list, u32 size)
          psx_gpu->viewport_start_y, psx_gpu->viewport_end_x,
          psx_gpu->viewport_end_y);
 #endif
+  			SET_Ex(3, list[0]);
   			break;
   
   		case 0xE4:
@@ -620,6 +696,7 @@ void gpu_parse(psx_gpu_struct *psx_gpu, u32 *list, u32 size)
          psx_gpu->viewport_start_y, psx_gpu->viewport_end_x,
          psx_gpu->viewport_end_y);
 #endif
+  			SET_Ex(4, list[0]);
   			break;
   
   		case 0xE5:
@@ -629,6 +706,7 @@ void gpu_parse(psx_gpu_struct *psx_gpu, u32 *list, u32 size)
         psx_gpu->offset_x = offset_x >> 21;
         psx_gpu->offset_y = offset_y >> 21; 
   
+  			SET_Ex(5, list[0]);
   			break;
   		}
 
@@ -648,6 +726,7 @@ void gpu_parse(psx_gpu_struct *psx_gpu, u32 *list, u32 size)
           psx_gpu->mask_msb = mask_msb;
         }
 
+  			SET_Ex(6, list[0]);
   			break;
       }
   
@@ -655,5 +734,12 @@ void gpu_parse(psx_gpu_struct *psx_gpu, u32 *list, u32 size)
   			break;
   	}
   }
+
+#ifdef PCSX
+breakloop:
+#endif
+  if (last_command != NULL)
+    *last_command = current_command;
+  return list - list_start;
 }
 
